/*
	File:		RegisteredPorts.cp

	Contains:	Simple tool to display registered ports

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

*/


#ifndef __OPENTPTGLOBALNEW__
#include <OpenTptGlobalNew.h>
#endif
#ifndef __OPENTRANSPORT__
#include <OpenTransport.h>
#endif
#ifndef __OPENTPTLINKS__
#include <OpenTptLinks.h>
#endif

#ifndef __STDIO__
#include <stdio.h>
#endif
#ifndef __EVENTS__
#include <Events.h>
#endif
#ifndef __DESK__
#include <Desk.h>
#endif


struct PortDef
{
	struct PortDef*	fNext;
	OTPortRecord	fRec;
};

const UInt16		kMaxTypes = 17;
static PortDef*		gPorts[kMaxTypes + 2];

void AddPortToList(OTPortRecord *, PortDef**);
void ShowIt();
void Cleanup();

/*******************************************************************************
** Doint
********************************************************************************/

void DoIt()
{
	size_t	idx;
	
	for( idx = 0;  idx < (sizeof(gPorts) / sizeof(PortDef*)); ++idx)
		gPorts[idx] = NULL;

	Boolean			found = false;
	OTPortRecord	record;
	
	idx = 0;
	while ( OTGetIndexedPort(&record, idx++) )
	{
		if ( OTGetDeviceTypeFromPortRef(record.fRef) <= kMaxTypes )
		{
			found = true;
			AddPortToList(&record, &gPorts[OTGetDeviceTypeFromPortRef(record.fRef)]);
		}
		else
		{
			found = true;
			AddPortToList(&record, &gPorts[kMaxTypes + 1]);
		}
	}
	if ( found )
		ShowIt();
	else
		printf("\n!!! No Registered Ports Found !!!\n");
	
	Cleanup();
}

/*******************************************************************************
** 
********************************************************************************/
			
void AddPortToList(OTPortRecord* pRec, PortDef** defList)
{
	PortDef*	pp		= *defList;
	PortDef*	prev	= NULL;
	PortDef*	newDef	= (PortDef*)OTAllocMem(sizeof(PortDef));
	UInt16		othera, otherb;
	
	if ( newDef == NULL )
		return;
		
	newDef->fRec = *pRec;
	
	while ( pp != NULL )
	{
		if ( OTGetSlotFromPortRef(pRec->fRef, &othera) < OTGetSlotFromPortRef(pp->fRec.fRef, &otherb) )
			break;
		else
		{
			prev = pp;
			pp = prev->fNext;
		}
	}
	newDef->fNext = pp;
	if ( prev == NULL )
	{
		*defList = newDef;
	}
	else
	{
		prev->fNext = newDef;
	}
}

/*******************************************************************************
** 
********************************************************************************/

void ShowPortInfo(OTPortRecord* pRec)
{
	UInt16	other;
	UInt16	slot = OTGetSlotFromPortRef(pRec->fRef, &other);
	UInt16	bus = OTGetBusTypeFromPortRef(pRec->fRef);
	
	static char* busNames[] =
	{
		"Unknown",
		"MthrBrd",
		"NuBus",
		"PCI",
		"GeoPort",
		"PCMCIA",
		"Bus6",
		"Bus7",
		"Bus8",
		"Bus9",
		"Bus10",
		"Bus11",
		"Bus12",
		"Bus13",
		"Bus14",
		"Bus15"
	};

	printf("%s [%08lX] (%s", pRec->fPortName, pRec->fRef, busNames[bus]);
	size_t len = strlen(pRec->fPortName) + 13 + strlen(busNames[bus]);

	if ( pRec->fInfoFlags & kOTPortIsAlias )
	{
		printf(", alias");
		len += 7;
	}
	
	printf(")");
	len += 1;
	while ( len++ < 27 )
		printf(" ");
	printf(" - %02d", slot);
	
	if ( other != 0 )
		printf(":%02u ", other);
	else
		printf("    ");
		
	printf("Frmng = %02X, Flags=%02X, Mod = %s",
		   pRec->fCapabilities & 0xff, pRec->fInfoFlags & 0xff,
		   pRec->fModuleName);
	printf("\n");
	if ( pRec->fResourceInfo[0] != 0 || pRec->fNumChildPorts != 0 )
	{
		printf("        ");
		if ( pRec->fResourceInfo[0] != 0 )
		{
			printf("RsrcInfo = \"%s\"", pRec->fResourceInfo);
			if ( pRec->fNumChildPorts != 0 )
				printf(",  ");
		}
		if ( pRec->fNumChildPorts != 0 )
		{
			printf("ChildPorts = ");
			size_t idx;
			for ( idx = 0; idx < pRec->fNumChildPorts; ++idx )
			{
				if ( idx != 0 )
					printf(", ");
				printf("%08lX", pRec->fChildPorts[idx]);
			}
		}
		printf("\n");
	}
}
	
	
/*******************************************************************************
** 
********************************************************************************/

void ShowIt()
{
	PortDef*	p;
	
	static char* MACtypes[] = {
				"<<unknown type(0)>>",
				"ADEV Devices",
				"MDEV Devices",
				"LocalTalk Devices",
				"IRTalk Devices",
				"TokenRing Devices",
				"ISDN Devices",
				"ATM Devices",
				"SMDS Devices",
				"Serial Devices",
				"Ethernet Devices",
				"SLIP Devices",
				"PPP Devices",
				"Modem Devices",
				"100MB Ethernet Devices",
				"FDDI Devices",
				"ATM LANE Devices",
				"ATM SNAP Devices"
				};
				
	for( int i = 0; i <= kMaxTypes; ++i)
	{
		p = gPorts[i];
		if ( p != NULL )
		{
			printf("\n%s\n", MACtypes[i]);
			while ( p )
			{
				ShowPortInfo( &p->fRec );
				p = p->fNext;
			}
		}
	}
	p = gPorts[kMaxTypes + 1];
	if ( p != NULL )
	{
			printf("\n%s\n", "Other Ports/Devices");
			while ( p )
			{
				ShowPortInfo( &p->fRec );
				p = p->fNext;
			}
	}
	fflush(stdout);
}

/*******************************************************************************
** 
********************************************************************************/

void Cleanup()
{
}

int main(int, char**) 
{
	OSStatus		err;
	RgnHandle		gRgn;
	
	InitGraf(&qd.thePort);						// initialize quickdraw so we can use regions
	gRgn = NewRgn();							// a region to use for WaitNextEvent

	if ( (err = InitOpenTransportUtilities()) != kOTNoError )
	{
		fprintf(stderr, "### ERROR: InitOpenTransportUtilities failed");
		return 1;
	}
	
	//
	// Run the test
	//
	DoIt();
	
	CloseOpenTransport();
		
	DisposeRgn(gRgn);

	return 0;
};
